unit FWMain;
{ Test Program for fast Sector-aligned stream write. On my disk, writing
the 1000 Blocks needed 14 seconds, compared to 27 seconds for normal
write when buffering is done by the operating system.
Makes for a realy smooth disk write for example in Video streams.


  Freeware (c) by Dieter Menne, Biomed Software, Tbingen
  Compuserve 100016,2125
  Internet 100016.2125@compuserve.com

 }

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms,
  StdCtrls;

  // ********************************************************
  // ********************************************************
const
  UseDisk='C:';    // Put name of your fastest disk here
  // ********************************************************
  // ********************************************************


type
  TForm1 = class(TForm)
    Button1: TButton;
    Memo1: TMemo;
    Label1: TLabel;
    Label2: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    procedure RunTest(Size: integer);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
uses Consts, FastStrm;



{ ------------------------------------ RunTest --------------------------------}
procedure TForm1.RunTest(Size: integer);
const
  RepCount=1024;    // Number of Blocks of this Size to write

var Written,FHandle,i: integer;
    Buf: PChar;
    Tim: TDateTime;
    FW: TFastWriteStream;
    FileName: String;
begin
  { Generate Data }
  Buf:=AllocMem(Size);
  for i:=0 to Size-1 do
    Buf[i]:=char(i mod 128);

  FileName:=UseDisk+'\AFastWriteTestFile.bin';

  { Test Fast Streaming }
  DeleteFile(FileName);
  FW:=TFastWriteStream.Create(FileName, Size);
  Label1.Caption:='Fast stream test running';
  Label1.Update;
  Tim:=Time;
  try
    for i:=1 to RepCount do   FW.Write(Buf^,Size);
  finally
    FW.Free;
  end;
  Tim:=(Time-Tim)*60*60*24;
  Memo1.Lines.Add(Format('Fast Stream %6.1f s, %12.3f MB/s',
         [Tim,Size*RepCount/(1024.*1024.*Tim)]));

  { Test Normal File Write; don't know why FileStream can't handle
  blocks > 1024 }
  DeleteFile(FileName);
  FHandle:=CreateFile(PChar(FileName),GENERIC_WRITE,0,nil,OPEN_ALWAYS,
                     FILE_ATTRIBUTE_NORMAL, 0);
  if FHandle < 0 then
      raise EFCreateError.CreateResFmt(SFCreateError, [FileName]);
  Label1.Caption:='Buffered write test running';
  Label1.Update;
  Tim:=Time;
  for i:=1 to RepCount do  // Should check shig
    WriteFile(FHandle,Buf^,Size,Written,nil);
  CloseHandle(FHandle);

  Tim:=(Time-Tim)*60*60*24;
  Memo1.Lines.Add(Format('WriteFile %6.1f s, %12.3f MB/s',
         [Tim,Size*RepCount/(1024.*1024.*Tim)]));
  Label1.Caption:='';
  FreeMem(Buf);
  DeleteFile(FileName);
end;

{ ----------------------------- Button1Click ---------------------------------}
procedure TForm1.Button1Click(Sender: TObject);
const NSectors=128;
var SectorSize: Integer;
    SizeStr: string;
begin
  Memo1.Clear;
  SectorSize:=GetSectorSize(UseDisk);
  SizeStr:=IntToStr(NSectors)+'*'+IntToStr(SectorSize);
  Memo1.Lines.Add('Sector Size '+IntToStr(SectorSize));
  Memo1.Lines.Add('Unaligned test with Size = '+SizeStr+' -1 Bytes');
  RunTest(NSectors*SectorSize-1);
  Memo1.Lines.Add('');
  Memo1.Lines.Add('Aligned   test with Size = '+SizeStr+' Bytes');
  RunTest(NSectors*SectorSize);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Label1.Caption:='';
end;

end.

